#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <mcs.h>
#ifndef OPTARG_DEFINED
#include <getopt.h>
#endif

void usage(char *pn) {
	fprintf(stderr,"usage: %s {-f|-t} table ... [file ...]\n",pn);
	fprintf(stderr,"\n");
	fprintf(stderr,"The program converts characters with several coding tables.\n");
	fprintf(stderr,"Any number of \"-f <tablename>\" and \"-t <tablename>\" may\n");
	fprintf(stderr,"be specified in any order, and will be applied sequentially.\n");
	fprintf(stderr,"The rest of parameters are file names to process.\n");
	fprintf(stderr,"If no files are specified, stdin is used.\n");
	fprintf(stderr,"-v<level> turns on debugging output.\n");
	fprintf(stderr,"\n");
	fprintf(stderr,"Example: \"translate -f win -t alt doc.txt\"\n");
	fprintf(stderr,"will convert the file doc.txt from `win' codetable to `alt'\n");
	fprintf(stderr,"codetable and output it to stdout\n");
}

int main(int argc, char *argv[])
{
	char buf[BUFSIZ];
	int i,len;
	int c;
	int verbose=0;
	FILE *fp;
	struct _trchain {
		struct _trchain *next;
		mcs_table_t *tab;
		int fr;
	} *trchain=NULL,**cur=&trchain;
	int status;

	while ((c=getopt(argc,argv,"v:f:t:h")) != EOF)
	switch (c) {
	case 'v':
		verbose=atoi(optarg);
		mcs_setverbose(verbose);
		break;
	case 'f':
	case 't':
		if (verbose) fprintf(stderr,"translate: %s\n",optarg);
		*cur=(struct _trchain *)malloc(sizeof(struct _trchain));
		(*cur)->next=NULL;
		(*cur)->tab=mcstab_byname(optarg,&status);
		(*cur)->fr=(c=='f')?1:0;
		cur=&((*cur)->next);
		if (status) fprintf (stderr,"%s: mcstab_byname returns %d\n",
					optarg,status);
		break;
	case 'h':
		usage(argv[0]);
		exit(0);
	default:
		usage(argv[0]);
		exit(1);
	}

	if (!trchain) {
		usage(argv[0]);
		exit(1);
	}

	do {
		if (argv[optind]) {
			if (verbose)
				fprintf(stderr,"processing %s\n",argv[optind]);
			if ((fp=fopen(argv[optind],"r")) == NULL) {
				perror(argv[optind++]);
				continue;
			}
			optind++;
		} else {
			if (verbose) fprintf(stderr,"processing stdin\n");
			fp=stdin;
		}

		while ((len=fread(buf,1,sizeof(buf),fp)) > 0)
		{
			for (cur=&trchain;*cur;cur=&((*cur)->next))
			if ((*cur)->fr) {
				for (i=0;i<len;i++) {
					buf[i]=MCS_IN((*cur)->tab,buf[i]);
				}
			} else {
				for (i=0;i<len;i++) {
					buf[i]=MCS_OUT((*cur)->tab,buf[i]);
				}
			}
			fwrite(buf,1,len,stdout);
		}

	} while (argv[optind]);

	fflush(stdout);
	return 0;
}
