/*	Copyright (C) 1997 Eugene G. Crosser and Sovam Teleport		*/

#ifndef LINT
static char *rscid="$Id: mcstab_byname.c,v 1.4 1999/11/26 17:03:22 crosser Exp $";
#endif

/*
	$Log: mcstab_byname.c,v $
	Revision 1.4  1999/11/26 17:03:22  crosser
	Configure for sunos4
	
	Revision 1.3  1999/08/26 15:19:25  crosser
	Allow changing tables directory
	
	Revision 1.2  1999/08/26 12:17:48  crosser
	Working automakified/libtoolized version
	
	Revision 1.1  1997/10/21 12:09:52  crosser
	Initial revision
	
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
# include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
#endif
#ifdef HAVE_LIMITS_H
# include <limits.h>
#endif
#include "mcs.h"
#include "mcs_cache.h"
#include "mcs_readmap.h"

#ifdef _REENTRANT
# include <pthread.h>
#endif

#ifdef DEBUG
#ifdef STDC_HEADERS
# include <stdio.h>
#endif

extern int mcs_verbose;
#endif

#ifndef TABDIR_ENV
# define TABDIR_ENV "MCS_TABDIR"
#endif

#ifndef TABDEST
# define TABDEST "/usr/lib/mcstables"
#endif

#ifndef PATH_MAX
#ifdef _POSIX_PATH_MAX
#define PATH_MAX _POSIX_PATH_MAX
#else
#define PATH_MAX 255
#endif
#endif

/* This is the heart of the whole thing.  Status of zero is returned if
   there is such table, and it was successfully loaded.  Status of plus
   one means that there is no table with this name.  This is usually
   tolerable.  Status of minus one means that something really bad
   happened, e.g. we could not resolve our name, or there is not enough
   memory for the table, or like.  This usually means that processing
   should be aborted. */

mcs_table_t *
mcstab_byname(char *nm,int *statusp)
{
	FILE *fp;
	char fname[PATH_MAX];
	mcs_table_t *ret_tab;
	char *tmpnm;
	char *tabdir;
	int status;
#ifdef _REENTRANT
	static pthread_mutex_t mutex;
#endif

	if ((nm == NULL) || (nm[0] == '\0')) {
		/* "no such table" is tolerable */
		if (statusp) *statusp=1;
		return &mcs_default_table;
	}
	if ((nm[0] == '/') && (nm[1] == '\0')) {
		/* slash used by mcsname_bysocket to denote severe error
		   -- set status to -1 to hush the user away */
		if (statusp) *statusp=-1;
		return &mcs_default_table;
	}
	if ((tabdir=getenv(TABDIR_ENV)) == NULL) tabdir=TABDEST;
	if ((strlen(tabdir)+strlen(nm)+2) > sizeof(fname)) {
		/* name too long - just invalid name - it is OK. */
		if (statusp) *statusp=1;
		return &mcs_default_table;
	}

	if ((ret_tab=mcs_findtable(nm,statusp))) {
#ifdef DEBUG
		if (mcs_verbose > 1)
			fprintf(stderr,"found(1) \"%s\" in cache\n",nm);
#endif
		return ret_tab;
	}

#ifdef _REENTRANT
	pthread_mutex_init(&mutex,(pthread_mutexattr_t*)0);
	pthread_mutex_lock(&mutex);

	if ((ret_tab=mcs_findtable(nm,statusp))) {
		pthread_mutex_unlock(&mutex);
		pthread_mutex_destroy(&mutex);
#ifdef DEBUG
		if (mcs_verbose > 1)
			fprintf(stderr,"found(2) \"%s\" in cache\n",nm);
#endif
		return ret_tab;
	}
#endif

	/* Go for real job now... */

	ret_tab=&mcs_default_table;

	tmpnm=(char*)malloc(strlen(nm)+1);
	if (tmpnm) {
		strcpy(tmpnm,nm);
	} else {
#ifdef DEBUG
		if (mcs_verbose) perror("allocating tmpnm");
#endif
		status=-1;
		goto exit;
	}
	ret_tab=(mcs_table_t*)malloc(sizeof(mcs_table_t));
	if (!ret_tab) {
#ifdef DEBUG
		if (mcs_verbose) perror("allocating ret_tab");
#endif
		status=-1;
		ret_tab=&mcs_default_table;
		free(tmpnm);
		goto exit;
	}
	sprintf(fname,"%s/%s",tabdir,nm);
#ifdef DEBUG
	if (mcs_verbose > 1) fprintf(stderr,"open \"%s\"\n",fname);
#endif
	fp=fopen(fname,"r");
	if (fp) {
		mcs_readmap(fp,ret_tab);
		fclose(fp);
		status=0;
	} else {
#ifdef DEBUG
		if (mcs_verbose) perror(fname);
#endif
		free(ret_tab);
		ret_tab=&mcs_default_table;
		status=1;
	}
	if (mcs_newtable(tmpnm,ret_tab,status)) {
		free(tmpnm);
		if(ret_tab != &mcs_default_table) free(ret_tab);
		ret_tab=&mcs_default_table;
		status=-1;
	}

exit:

#ifdef _REENTRANT
	pthread_mutex_unlock(&mutex);
	pthread_mutex_destroy(&mutex);
#endif
	if (statusp) *statusp=status;
	return ret_tab;
}
